# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

from base64 import urlsafe_b64decode
from tempfile import NamedTemporaryFile
from subprocess import CalledProcessError
import os
import sys

from django.conf import settings
from django.test.testcases import TestCase
import six

from m3_auth.backends.oauth2.signers import PKCS7Signer
from tests.testapp.tests.utils import MockCheckOutput


class TestPKCS7Signer(TestCase):

    """Проверка класса PCKS7Signer."""

    def setUp(self):
        super(TestPKCS7Signer, self).setUp()
        self.signer = PKCS7Signer(
            os.path.join(settings.CERTS_PATH, 'sp.crt'),
            os.path.join(settings.CERTS_PATH, 'sp.key')
        )
        self.params_to_sign = {
            'scope': 'some scope',
            'timestamp': 'timestamp',
            'client_id': '123',
            'state': 'some state'
        }

    def test_method_call(self):
        """Проверка подписи."""
        sys.stdout.write('\n')
        # Получаем текст для подписывания
        text_to_sign = self.signer._get_text_to_sign(self.params_to_sign)
        # Подписываем
        signature = self.signer(self.params_to_sign)
        self.assertTrue(signature)

        with NamedTemporaryFile('wb', delete=False) as fs:
            fs.write(urlsafe_b64decode(signature))
            signature_path = fs.name
        with NamedTemporaryFile('wb', delete=False) as fs:
            fs.write(six.b(text_to_sign))
            message_path = fs.name

        command = (
            'openssl smime -verify -inform DER -in {sign} '
            '-content {content} -noverify -certfile {cert} > /dev/null'
        )

        result = os.system(command.format(
            sign=signature_path,
            content=message_path,
            cert=self.signer.certificate_file
        ))

        self.assertEqual(result, 0, 'Signature verification failed!')
        os.remove(signature_path)
        os.remove(message_path)

    def test_check_openssl(self):
        """Проверка метода _check_openssl."""

        def raise_called_process_error(*args, **kwargs):
            raise CalledProcessError('test', 'test')

        with MockCheckOutput(raise_called_process_error):
            self.assertRaises(OSError, PKCS7Signer._check_openssl)
