# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

from datetime import date

from django.test.testcases import TestCase

from m3_auth.fields import DateField
from m3_auth.fields import SimpleField


class Person(object):

    """Для проверки дескрипторов."""

    GENDER = (
        (1, 'Мужской'),
        (2, 'Женский')
    )

    user_id = SimpleField('user_id')

    surname = SimpleField('surname')
    first_name = SimpleField('first_name')
    patronymic = SimpleField('patronymic', default='')
    date_of_birth = DateField('date_of_birth', '%Y-%m-%d')
    gender = SimpleField('gender', choices=GENDER)

    phone = SimpleField('contacts.phone')

    weight = SimpleField('weight', default=int)

    def __init__(self, data):
        self.data = data


class FieldsTestCase(TestCase):

    """Проверка модуля fields."""

    def setUp(self):
        super(FieldsTestCase, self).setUp()
        self.john = Person({
            'user_id': 1,
            'surname': 'Smith',
            'first_name': 'John',
            'date_of_birth': '1990-01-01',
            'gender': 1,
            'contacts': {
                'phone': '+7 (917) 222-33-44'
            }
        })
        self.emma = Person({
            'user_id': 2,
            'surname': 'Smith',
            'first_name': 'Emma',
            'patronymic': 'None',
            'gender': 2,
            'weight': 50
        })

    def test_fields(self):
        """Проверка дескрипторов."""
        self.assertEqual(self.john.user_id, 1)
        self.assertEqual(self.john.surname, 'Smith')
        self.assertEqual(self.john.first_name, 'John')
        self.assertEqual(self.john.patronymic, '')
        self.assertEqual(self.john.date_of_birth, date(1990, 1, 1))
        self.assertEqual(self.john.gender, 'Мужской')
        self.assertEqual(self.john.phone, '+7 (917) 222-33-44')
        self.assertEqual(self.john.weight, 0)

        self.assertEqual(self.emma.user_id, 2)
        self.assertEqual(self.emma.surname, 'Smith')
        self.assertEqual(self.emma.first_name, 'Emma')
        self.assertEqual(self.emma.patronymic, 'None')
        self.assertIs(self.emma.date_of_birth, None)
        self.assertEqual(self.emma.gender, 'Женский')
        self.assertIs(self.emma.phone, None)
        self.assertEqual(self.emma.weight, 50)
