# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

from django.contrib.sessions.middleware import SessionMiddleware
from django.test.client import RequestFactory
from django.test.testcases import TestCase

from m3_auth.proxy import SessionProxy


class ProxyTestCase(TestCase):

    """Проверка модуля proxy.py."""

    def setUp(self):
        super(ProxyTestCase, self).setUp()
        self.request = RequestFactory().post('/')
        SessionMiddleware().process_request(self.request)

    def test_session_proxy(self):
        """Проверка класса SessionProxy."""
        prefix = 'test_prefix'
        session_proxy = SessionProxy(self.request, prefix)

        self.assertTrue(session_proxy)

        # Проверка атрибута user_id
        self.assertIs(session_proxy.remote_user_id, None)
        session_proxy.remote_user_id = 1
        self.assertEqual(
            session_proxy.remote_user_id,
            self.request.session['{}:user_id'.format(prefix)]
        )

        # Проверка атрибута backend
        self.assertIs(session_proxy.backend, None)
        session_proxy.backend = 'django'
        self.assertEqual(
            session_proxy.backend,
            self.request.session['{}:backend'.format(prefix)]
        )

        # Проверка атрибута user_ids_to_select
        self.assertIs(session_proxy.user_ids_to_select, tuple())
        session_proxy.user_ids_to_select = (1, 2, 3)
        self.assertEqual(
            session_proxy.user_ids_to_select,
            self.request.session['{}:user_ids_to_select'.format(prefix)]
        )

        # Проверка атрибута token
        self.assertIs(session_proxy.token, None)
        session_proxy.token = 'this is token.'
        self.assertEqual(
            session_proxy.token,
            self.request.session['{}:token'.format(prefix)]
        )
